//============================================================================
// CFA-10000-12864A-C Demonstration code
// LOW_LCD.C: Low-level lcd display routines.
// Samsung KS0107+KS0108 (S6B0107+S6B0108) LCD Controller
// Atmel ATMega32 processor @ 16MHz
// WinAVR / AVR GCC + Atmel AVR Studio
// Written by Brent A. Crosby
// Copyright 2005, Crystalfontz America, Inc. http://www.crystalfontz.com
//============================================================================
#include <avr/io.h>
#include <util/delay.h>
#include <avr/iom2561.h>
#include <avr/pgmspace.h>
#include <string.h>
#include "utils.h"
#include "low_lcd.h"
#include "logo_screen.h"
#include "MMC_SD.h"

//============================================================================
//This is the display memory.
volatile unsigned char display[32][32];  //1024 bytes
unsigned char displaytext[2][32]={{'0','1','2','3','4','5','6','7','8','9','0','A','B','C','D','E','F','W','X','Y','Z',1,2,3,4,5,6,7,8,9,10,11,12,13,},
                                  {'G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V',14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,}};
unsigned int chinesetext[2][16]={{0xA6B7,0xA6B6,0xA6B5,0xA6B4,0xA6B3,0xA6B2,0xA6B1,0xA6B0,  0xA757,0xA756,0xA755,0xA754,0xA753,0xA752,0xA751,0xA750},
																 {0xA6BF,0xA6BE,0xA6BD,0xA6BC,0xA6BB,0xA6BA,0xA6B9,0xA6B8,  0xA75F,0xA75E,0xA75D,0xA75C,0xA75B,0xA75A,0xA759,0xA758}};
//============================================================================
volatile unsigned char display_status;

//============================================================================
void write_command(unsigned char data)
{
  //Go back to port A as an output
  LCD_DATA_DDR = LCD_DATA_ALL_OUTPUTS;
  LCD_DATA_PORT = data;

  //We need to wait until both controller's busy bits are clear.
  //Talk to the status (instruction) register.
  CLR_RS;
  //Tell the LCD that we are writing to it.
  CLR_RW; 
	
  SET_E;  // is SCK
  CLR_E;
  _delay_us(850);
   SET_E;
}

//----------------------------------------------------------------------------
void write_data(unsigned char data)
{
  //Go back to port A as an output
  LCD_DATA_DDR = LCD_DATA_ALL_OUTPUTS;
  //Write the data to the output latches of port A
  LCD_DATA_PORT = data;
  //Talk to the data register.
  SET_RS;
  //Tell the LCD that we are writing to it.
  CLR_RW;
  SET_E;
  CLR_E;
  _delay_us(850);
  SET_E;
}
//----------------------------------------------------------------------------
void write_data_16(unsigned int data)
{
  //Go back to port A as an output
  LCD_DATA_DDR = LCD_DATA_ALL_OUTPUTS;
  //Write the data to the output latches of port A
  LCD_DATA_PORT = data>>8;
  //Talk to the data register.
  SET_RS;
  //Tell the LCD that we are writing to it.
  CLR_RW;
  SET_E;
  CLR_E;
  _delay_us(850);
  SET_E;
  LCD_DATA_PORT = data;
  CLR_E;
  _delay_us(850);
  SET_E;
}
//----------------------------------------------------------------------------
void show_text(void)
{
     register unsigned char page_address;
     register unsigned char column_address;

     write_command(0x38);
     write_command(0x38);
     write_command(0x38);
     write_command(0x38);
//     write_command(0x08);
     write_command(0x0A);  // cursor on
     write_command(0x01);
     write_command(0x06);
     write_command(0x0F);
     write_command(0x01);



	for (page_address = 0; page_address < 2; page_address++)
	{

		if(page_address==1)
			write_command(0x90);

		for (column_address = 0; column_address <33; column_address++)
		{
			write_data(displaytext[page_address][column_address]);
		}
	}
}

//----------------------------------------------------------------------------
void show_chinese_text(void)
{
     register unsigned char page_address;
     register unsigned char column_address;

     write_command(0x38);
     write_command(0x38);
     write_command(0x38);
     write_command(0x38);
     write_command(0x08);
//     write_command(0x0A);  // cursor on
     write_command(0x01);
     write_command(0x06);
     write_command(0x0F);
     write_command(0x01);

			write_command(0x80);

	for (page_address = 0; page_address < 2; page_address++)
	{

		if(page_address==1)
			write_command(0x90);

		for (column_address = 0; column_address <17; column_address++)
		{
			write_data_16(chinesetext[page_address][column_address]);
		}
	}
}

//============================================================================
void UpdateLCD(void)
{
	register volatile unsigned char *data;
	register unsigned char page_address;
	register unsigned char column_address;

	data = &display[0][0];


	//Initialize our source data pointers.
	for (page_address = 0; page_address < 32; page_address++)
	{
		//Set Y address
		write_command(page_address | 0x80);
		//Set X address 0 (right to left)
		write_command(0x80);

		for (column_address = 0; column_address <32; column_address++)
		{
			write_data(*data++);
		}
	}
}

void ResetLCD(void)
{
     CLR_RESET;
     _delay_us(6500);
  _delay_us(6500);    
  _delay_us(6500);
  _delay_us(6500);
  _delay_us(6500);
  _delay_us(6500);
     SET_RESET;
}

//============================================================================
void InitializeLCD(void)
{
	/* Take the LCD out of reset (if reset was low)
	*/
	SET_RESET;

	_delay_us(6500);

	//Display on
	write_command(0x30);
	write_command(0x0C);
	write_command(0x01);
	write_command(0x06);
	write_command(0x30);
	write_command(0x32);
	write_command(0x36);

}
//============================================================================
